package com.sagacloud.hbase;
/*
 * Author: Jxing
 * Create Time: 2019/3/13
 */

import com.alibaba.fastjson.*;
import com.zillion.database.agent.*;
import com.zillion.database.util.TableWrapper_HBase;
import com.zillion.util.common.ByteToObject;
import com.zillion.util.table.HTableUtil;
import com.zillion.util.table.ZillionTableIndex;
import com.zillion.util.table.ZillionTableSchema;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.Filter;
import org.apache.hadoop.hbase.filter.FilterList;
import org.apache.hadoop.hbase.filter.MultiRowRangeFilter;
import org.apache.hadoop.hbase.util.Bytes;
import org.zillion.util.common.MyException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class HBaseScanUtil {
    /**
     * @param
     * @UpdateDate: 2018/9/14 10:51
     * @Description: scan主方法
     * @return: split_child : 无分表时和tablename相同
     * Criteria，Criterias :  查询条件  ,只能传一个,另外一个传null
     */
    public static ResultScanner getScan(ZillionAgent agent, String database, String tablename, String split_child, JSONObject Criteria, JSONArray Criterias) throws Exception {
        ZillionTableSchema schema = agent.container.repositoryMap.get(database).GetTable(tablename);
        if (Criteria != null && Criterias != null) {
            throw new MyException("Criteria and Criterias not all null");
        }
        if (Criteria != null) {
            ZillionQueryCheckUtil.CheckCriteria(schema, Criteria);
        } else {
            Criteria = (JSONObject) Criterias.get(0);
            for (int i = 0; i < Criterias.size(); i++) {
                JSONObject CriteriaInner = (JSONObject) Criterias.get(i);
                ZillionQueryCheckUtil.CheckCriteria(schema, CriteriaInner);
            }
        }
        Scan scan = new Scan();
        //设置获取列
        scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes("v"));
        if (schema.table_type == null || schema.table_type.equals("index_data")
                || (schema.table_type.equals("split_month") || schema.table_type.equals("split_year"))) {
            for (int i = 0; i < schema.OtherColumn.length; i++) {
                String column_name = schema.OtherColumn[i];
                scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
            }
        } else if (schema.table_type.equals("all_column")) {
            String[] Columns = schema.ColumnMap.keySet().toArray(new String[0]);
            for (int ii = 0; ii < Columns.length; ii++) {
                String column_name = Columns[ii];
                scan.addColumn(ByteToObject.toBytes("f"), ByteToObject.toBytes(column_name));
            }
        }
        JSONArray Cris = new JSONArray();
        Cris.add(Criteria);
        String indexname = HbaseScanIndex.indexname(schema, "select", new JSONArray(), Cris);
        TableWrapper_HBase wrapper = hTableWrapper(indexname, schema, agent, split_child);
        ZillionTableIndex index = HbaseScanIndex.zillionTableIndex(schema, "select", new JSONArray(), Cris);

        //设置过滤
        List<ZillionCriteria> criteria_keyList = new ArrayList<>();
        ZillionCriteria criteria_key = get_criteria_key(schema, index, Criteria);
        if (criteria_key != null) {
            criteria_keyList.add(criteria_key);
            if (Criterias != null && Criterias.size() > 1) {
                for (int i = 1; i < Criterias.size(); i++) {
                    JSONObject CriteriaInner = (JSONObject) Criterias.get(i);
                    ZillionCriteria criteria_tmp = get_criteria_key(schema, index, CriteriaInner);
                    if (criteria_tmp == null) {
                        throw new MyException("ZillionAgent: " + (i + 1) + " Criteria must follow key's rule!");
                    }
                    criteria_keyList.add(criteria_tmp);
                }
            }
        } else {
            if (Criterias != null && Criterias.size() > 1) {
                throw new MyException("ZillionAgent: " + "table of type all_column Criterias must follow rule!");
            }
            ZillionCriteria_all_column zillionCriteria_all_column = ZillionMatchUtil.CriteriaMatch_all_column_best(schema, Criteria);
            if (zillionCriteria_all_column.isKey) {
                criteria_key = zillionCriteria_all_column.criteria;
                criteria_keyList.add(criteria_key);
            }
        }
        boolean exist_match_all = false;
        List<MultiRowRangeFilter.RowRange> rangeList = new ArrayList<MultiRowRangeFilter.RowRange>();
        for (ZillionCriteria criteria_tmp : criteria_keyList) {
            if (criteria_tmp.matched == 0) {
                exist_match_all = true;
                break;
            }
            if (criteria_tmp.bytes_lt_include && Bytes.compareTo(criteria_tmp.bytes_gt, criteria_tmp.bytes_lt) == 0) {
                throw new MyException("ZillionAgent: " + "criteria_key lt_include & gt is equal to lt!!!");
            }
            rangeList.add(new MultiRowRangeFilter.RowRange(criteria_tmp.bytes_gt, criteria_tmp.bytes_gt_include, criteria_tmp.bytes_lt, criteria_tmp.bytes_lt_include));
        }
        if (exist_match_all) {
            if (criteria_key.filterList.size() > 0) {
                if (criteria_key.filterList.size() == 1) {
                    scan.setFilter(criteria_key.filterList.get(0));
                } else {
                    FilterList filterList = new FilterList(FilterList.Operator.MUST_PASS_ALL);
                    for (Filter filter : criteria_key.filterList) {
                        filterList.addFilter(filter);
                    }
                    scan.setFilter(filterList);
                }
            }
        } else {
            Filter rowFilter = new MultiRowRangeFilter(rangeList);
            if (criteria_key.filterList.size() > 0) {
                FilterList filterList = new FilterList(FilterList.Operator.MUST_PASS_ALL);
                filterList.addFilter(rowFilter);
                for (Filter filter : criteria_key.filterList) {
                    filterList.addFilter(filter);
                }
                scan.setFilter(filterList);
            } else {
                scan.setFilter(rowFilter);
            }
        }

        ResultScanner scanner = wrapper.htable.getScanner(scan);
        return scanner;
    }

    private static TableWrapper_HBase hTableWrapper(String indexname, ZillionTableSchema schema, ZillionAgent agent, String split_child) throws Exception {
        TableWrapper_HBase wrapper = null;
        if (indexname != null) {
            for (int i = 0; i < schema.Indexes.size(); i++) {
                ZillionTableIndex index1 = schema.Indexes.get(i);
                if (index1.index_name.equals(indexname)) {
                    wrapper = (TableWrapper_HBase) index1.connect(agent.container, split_child, false);
                    break;
                }
            }
        } else {
            wrapper = (TableWrapper_HBase) schema.connect(agent.container, split_child, false);
        }
        return wrapper;
    }

    private static ZillionCriteria get_criteria_key(ZillionTableSchema schema, ZillionTableIndex index, JSONObject Criteria) throws IOException {
        ZillionCriteria criteria_key;
        if (index == null) {
            criteria_key = ZillionMatchUtil.CriteriaMatch(schema.ColumnMap, schema.Key, Criteria, true);
        } else {
            criteria_key = ZillionMatchUtil.CriteriaMatch(schema.ColumnMap, index.Key, Criteria, true);
        }
        return criteria_key;
    }

    public static JSONObject Convert(ZillionTableSchema schema, ZillionTableIndex index, Result rs) throws IOException {
        JSONObject resultObject;
        if (index != null) {
            resultObject = HTableUtil.Convert_index_data(schema, index, rs.getRow(), rs.listCells());
        } else {
            resultObject = HTableUtil.Convert(schema, rs.getRow(), rs.listCells());
        }
        return resultObject;
    }
}
